@@ -1,3 +1,4 @@
+from .claims import Claims
 from .claims_builder import ClaimsBuilder, claims
 from .cose import COSE
 from .cose_key import COSEKey
@@ -8,12 +9,13 @@ from .cwt import (
     encode_and_encrypt,
     encode_and_mac,
     encode_and_sign,
+    set_private_claim_names,
 )
 from .exceptions import CWTError, DecodeError, EncodeError, VerifyError
 from .key_builder import KeyBuilder, cose_key
 from .recipient import Recipient
 
-__version__ = "0.5.0"
+__version__ = "0.6.0"
 __title__ = "cwt"
 __description__ = "A Python implementation of CWT/COSE"
 __url__ = "https://python-cwt.readthedocs.io"
@@ -29,8 +31,10 @@ __all__ = [
     "encode_and_sign",
     "encode_and_encrypt",
     "decode",
+    "set_private_claim_names",
     "CWT",
     "COSE",
+    "Claims",
     "ClaimsBuilder",
     "COSEKey",
     "KeyBuilder",
