@@ -350,9 +350,9 @@ class TestSample:
 
     def test_sample_readme_nested_cwt_old(self):
         with open(key_path("private_key_es256.pem")) as key_file:
-            private_key = cose_key.from_pem(key_file.read())
+            private_key = cose_key.from_pem(key_file.read(), kid="01")
         with open(key_path("public_key_es256.pem")) as key_file:
-            public_key = cose_key.from_pem(key_file.read())
+            public_key = cose_key.from_pem(key_file.read(), kid="01")
 
         encoded = cwt.encode_and_sign(
             claims.from_json(
@@ -363,13 +363,31 @@ class TestSample:
 
         nonce = token_bytes(13)
         mysecret = token_bytes(32)
-        enc_key = cose_key.from_symmetric_key(mysecret, alg="AES-CCM-16-64-256")
+        enc_key = cose_key.from_symmetric_key(
+            mysecret, alg="AES-CCM-16-64-256", kid="02"
+        )
         nested = cwt.encode_and_encrypt(encoded, enc_key, nonce=nonce)
 
         decoded = cwt.decode(nested, [enc_key, public_key])
         assert 1 in decoded and decoded[1] == "coaps://as.example"
 
     def test_sample_readme_nested_cwt(self):
+        with open(key_path("private_key_es256.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read(), kid="01")
+        with open(key_path("public_key_es256.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read(), kid="01")
+
+        token = cwt.encode(
+            {"iss": "coaps://as.example", "sub": "dajiaji", "cti": "123"}, private_key
+        )
+
+        enc_key = cose_key.from_symmetric_key(alg="ChaCha20/Poly1305", kid="02")
+        nested = cwt.encode(token, enc_key)
+
+        decoded = cwt.decode(nested, [enc_key, public_key])
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
+
+    def test_sample_readme_nested_cwt_without_kid(self):
         with open(key_path("private_key_es256.pem")) as key_file:
             private_key = cose_key.from_pem(key_file.read())
         with open(key_path("public_key_es256.pem")) as key_file:
@@ -576,6 +594,94 @@ class TestSample:
         assert 8 in decoded and isinstance(decoded[8], dict)
         assert 3 in decoded[8] and decoded[8][3] == b"pop-key-id-of-cwt-presenter"
 
+    def test_sample_readme_cwt_with_user_defined_claims(self):
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        token = cwt.encode(
+            {
+                1: "coaps://as.example",  # iss
+                2: "dajiaji",  # sub
+                7: b"123",  # cti
+                -70001: "foo",
+                -70002: ["bar"],
+                -70003: {"baz": "qux"},
+                -70004: 123,
+            },
+            private_key,
+        )
+        raw = cwt.decode(token, public_key)
+        assert raw[-70001] == "foo"
+        assert isinstance(raw[-70002], list)
+        assert raw[-70002][0] == "bar"
+        assert isinstance(raw[-70003], dict)
+        assert raw[-70003]["baz"] == "qux"
+        assert raw[-70004] == 123
+        readable = claims.from_dict(raw)
+        assert readable.get(-70001) == "foo"
+        assert readable.get(-70002)[0] == "bar"
+        assert readable.get(-70003)["baz"] == "qux"
+        assert readable.get(-70004) == 123
+
+    def test_sample_readme_cwt_with_user_defined_claims_readable(self):
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read())
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key = cose_key.from_pem(key_file.read())
+        cwt.set_private_claim_names(
+            {
+                "ext_1": -70001,
+                "ext_2": -70002,
+                "ext_3": -70003,
+                "ext_4": -70004,
+            }
+        )
+        token = cwt.encode(
+            {
+                "iss": "coaps://as.example",
+                "sub": "dajiaji",
+                "cti": b"123",
+                "ext_1": "foo",
+                "ext_2": ["bar"],
+                "ext_3": {"baz": "qux"},
+                "ext_4": 123,
+            },
+            private_key,
+        )
+        claims.set_private_claim_names(
+            {
+                "ext_1": -70001,
+                "ext_2": -70002,
+                "ext_3": -70003,
+                "ext_4": -70004,
+            }
+        )
+        raw = cwt.decode(token, public_key)
+        readable = claims.from_dict(raw)
+        assert readable.get("ext_1") == "foo"
+        assert readable.get("ext_2")[0] == "bar"
+        assert readable.get("ext_3")["baz"] == "qux"
+        assert readable.get("ext_4") == 123
+
+    def test_sample_readme_decode_with_multiple_keys(self):
+        with open(key_path("public_key_es256.pem")) as key_file:
+            public_key_1 = cose_key.from_pem(key_file.read(), kid="01")
+        with open(key_path("public_key_ed25519.pem")) as key_file:
+            public_key_2 = cose_key.from_pem(key_file.read(), kid="02")
+        with open(key_path("private_key_ed25519.pem")) as key_file:
+            private_key = cose_key.from_pem(key_file.read(), kid="02")
+        token = cwt.encode(
+            {
+                "iss": "coaps://as.example",
+                "sub": "dajiaji",
+                "cti": b"123",
+            },
+            private_key,
+        )
+        decoded = cwt.decode(token, [public_key_1, public_key_2])
+        assert 1 in decoded and decoded[1] == "coaps://as.example"
+
     def test_sample_rfc8392_a3(self):
         key = cose_key.from_bytes(bytes.fromhex(SAMPLE_COSE_KEY_RFC8392_A2_3))
         encoded = bytes.fromhex(SAMPLE_CWT_RFC8392_A3)
